import javax.imageio.*;
import java.awt.image.*;
import java.io.*;

public class Comp202ImageIO {

    public static BufferedImage getBufferedImageFromArray(int[] pixelArray, int width, int height) {
	BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
	bufferedImage.setRGB(0,0,width,height,pixelArray,0,width);
	return bufferedImage;
    }

    public static int getImageWidth(String inputFile) {
	BufferedImage img = null;
	try {
	    img = ImageIO.read(new File(inputFile));
	}
	catch (IOException e) {
	    System.out.println("There was an IO exception. Most likely the file you are trying to load was not found");
	    e.printStackTrace();
	    System.out.println(e.getMessage());
	}

	return img.getWidth();
    }

    public static int getImageHeight(String inputFile) {
	BufferedImage img = null;
	try {
	    img = ImageIO.read(new File(inputFile));
	}
	catch (IOException e) {
	    System.out.println("There was an IO exception. Most likely the file you are trying to load was not found");
	    e.printStackTrace();
	    System.out.println(e.getMessage());
	}

	return img.getHeight();
    }

    public static int[][] loadImage(String inputFile) {
	BufferedImage img = null;
	try {
	    img = ImageIO.read(new File(inputFile));
	}
	catch (IOException e) {
	    System.out.println("There was an IO exception. Most likely the file you are trying to load was not found");
	    e.printStackTrace();
	    System.out.println(e.getMessage());
	}
	
	int[] original = img.getRGB(0,0,img.getWidth(),img.getHeight(),null,0,img.getWidth());

	int[][] finalImage = new int[4][original.length];

	for (int i=0; i < original.length; i++) {
		int[] byteArray = convertArgbToByteArray(original[i]);
		finalImage[0][i] = byteArray[0];
		finalImage[1][i] = byteArray[1];
		finalImage[2][i] = byteArray[2];
		finalImage[3][i] = byteArray[3];
	}

	return finalImage;
    }

    public static BufferedImage getBufferedImage(int[][] imageData, int height, int width) {
	//first flatten the data:
	int[] pixelsFlattened = new int[imageData[0].length];
	for (int i=0; i < imageData[0].length; i++) {
		pixelsFlattened[i] = convertARGBToInt(imageData[0][i], imageData[1][i], imageData[2][i], imageData[3][i]);
	}

	BufferedImage bufferedImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
	bufferedImage.setRGB(0,0,width,height,pixelsFlattened,0,width);
	return bufferedImage;
    }

    public static void savePng(int[][] imageData, int height, int width, String filename) {
	try {
		BufferedImage bufferedImage = getBufferedImage(imageData,height, width);
		ImageIO.write(bufferedImage, "png", new File(filename));
	}
	catch (IOException e) {
		System.out.println("Unable to save file. Perhaps the location you are trying to save to does not exist or is not readable?");
		System.out.println(e.getMessage());
		e.printStackTrace();
	}
    }

    private static int convertARGBToInt(int alpha, int red, int green, int blue) {
	return (alpha << 24) + (red << 16) + (green << 8) + blue;
    }

    private static int convertByteArrayToInt(int[] argbArray) {
	return (argbArray[0] << 24) + (argbArray[1] << 16) + (argbArray[2] << 8) + argbArray[3];
    }

    private static int[] convertArgbToByteArray(int original) {
	int[] argbArray = new int[4];
	argbArray[0] = getAlpha(original);
	argbArray[1] = getRed(original);
	argbArray[2] = getGreen(original);
	argbArray[3] = getBlue(original);
	return argbArray;
    }

    private static int getAlpha(int original) {
	return original >> 24;
    }

    private static int getRed(int original) {
	return (original >> 16) & 0xFF;
    }

    private static int getGreen(int original) {
	return (original >> 8) & 0xFF;
    }

    private static int getBlue(int original) {
	return (original & 0xFF);
    }
}